@@ -1,10 +1,5 @@
 from __future__ import absolute_import, print_function
 
-import logging
-
-logging.basicConfig(format='%(levelname)s:%(funcName)s:%(message)s',
-                    level=logging.DEBUG)
-
 
 def suite():
     from M2Crypto import m2  # noqa
@@ -25,6 +20,7 @@ def suite():
         return mod
 
     modules_to_test = [
+        'tests.test_aes',
         'tests.test_asn1',
         'tests.test_bio',
         'tests.test_bio_membuf',
@@ -38,7 +34,6 @@ def suite():
         'tests.test_engine',
         'tests.test_evp',
         'tests.test_obj',
-        'tests.test_pgp',
         'tests.test_rand',
         'tests.test_rc4',
         'tests.test_rsa',
@@ -58,7 +53,8 @@ def suite():
     for module in map(my_import, modules_to_test):
         alltests.addTest(module.suite())
 
-    print('Version of OpenSSL is {0:x}'.format(m2.OPENSSL_VERSION_NUMBER))
+    print('Version of OpenSSL is {0:x} ({1:s})'.format(m2.OPENSSL_VERSION_NUMBER,
+            m2.OPENSSL_VERSION_TEXT))
 
     return alltests
 
