@@ -30,14 +30,10 @@ import sys
 import tempfile
 import time
 import warnings
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
 
 from M2Crypto import (Err, Rand, SSL, X509, ftpslib, httpslib, m2, m2urllib,
-                      m2urllib2, m2xmlrpclib, six, util)
-from tests import plat_fedora
+                      m2urllib2, m2xmlrpclib, py27plus, six)
+from tests import unittest
 from tests.fips import fips_mode
 
 log = logging.getLogger('test_SSL')
@@ -78,7 +74,7 @@ def verify_cb_new_function(ok, store):
     return 1
 
 
-class VerifyCB:
+class VerifyCB(object):
     def __call__(self, ok, store):
         return verify_cb_new_function(ok, store)
 
@@ -120,7 +116,7 @@ class BaseSSLClientTestCase(unittest.TestCase):
     def stop_server(self, pid):
         pid.terminate()
         out, err = pid.communicate()
-        return util.py3str(out), util.py3str(err)
+        return six.ensure_text(out), six.ensure_text(err)
 
     def http_get(self, s):
         s.send(b'GET / HTTP/1.0\n\n')
@@ -133,7 +129,7 @@ class BaseSSLClientTestCase(unittest.TestCase):
             except SSL.SSLError:  # s_server throws an 'unexpected eof'...
                 break
             resp = resp + r
-        return util.py3str(resp)
+        return six.ensure_text(resp)
 
     def setUp(self):
         self.srv_host = srv_host
@@ -168,7 +164,7 @@ class HttpslibSSLClientTestCase(BaseSSLClientTestCase):
             c.close()
         finally:
             self.stop_server(pid)
-        self.assertIn('s_server -quiet -www', util.py3str(data))
+        self.assertIn('s_server -quiet -www', six.ensure_text(data))
 
     def test_HTTPSConnection_resume_session(self):
         pid = self.start_server(self.args)
@@ -199,7 +195,7 @@ class HttpslibSSLClientTestCase(BaseSSLClientTestCase):
             c2.request('GET', '/')
             ses2 = c2.get_session()
             t2 = ses2.as_text()
-            data = util.py3str(c2.getresponse().read())
+            data = six.ensure_text(c2.getresponse().read())
             c.close()
             c2.close()
             self.assertEqual(t, t2, "Sessions did not match")
@@ -216,7 +212,7 @@ class HttpslibSSLClientTestCase(BaseSSLClientTestCase):
             c = httpslib.HTTPSConnection(srv_host, self.srv_port,
                                          ssl_context=self.ctx)
             c.request('GET', '/')
-            data = util.py3str(c.getresponse().read())
+            data = six.ensure_text(c.getresponse().read())
             c.close()
         finally:
             self.stop_server(pid)
@@ -418,30 +414,6 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             self.stop_server(pid)
         self.assertIn('s_server -quiet -www', data)
 
-    # TLS is required in FIPS mode
-    @unittest.skipIf(fips_mode, "Can't be run in FIPS mode")
-    @unittest.skipUnless(hasattr(m2, "sslv2_method"),
-                         "This platform doesn't support SSLv2")
-    def test_sslv23_weak_crypto(self):
-        self.args = self.args + ['-ssl2']
-        pid = self.start_server(self.args)
-        try:
-            ctx = SSL.Context('sslv23', weak_crypto=1)
-            s = SSL.Connection(ctx)
-            # SSLv2 ciphers disabled by default in newer OpenSSL
-            if plat_fedora and m2.OPENSSL_VERSION_NUMBER < 0x10000000:
-                s.connect(self.srv_addr)
-                self.assertEqual(s.get_version(), 'SSLv2')
-            else:
-                with self.assertRaises(SSL.SSLError):
-                    s.connect(self.srv_addr)
-            s.close()
-        except Exception as ex:
-            print(('Caught exception %s' % ex))
-            raise
-        finally:
-            self.stop_server(pid)
-
     def test_cipher_mismatch(self):
         self.args = self.args + ['-cipher', 'AES256-SHA']
         pid = self.start_server(self.args)
@@ -463,7 +435,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             ctx = SSL.Context()
             s = SSL.Connection(ctx)
             s.set_cipher_list('EXP-RC2-MD5')
-            with six.assertRaisesRegex(self, SSL.SSLError, 'no ciphers available'):
+            with six.assertRaisesRegex(self, SSL.SSLError,
+                                       'no ciphers available'):
                 s.connect(self.srv_addr)
             s.close()
         finally:
@@ -877,6 +850,53 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             self.stop_server(pid)
         self.assertIn('s_server -quiet -www', data)
 
+    def test_ssl_connection_free(self):
+        pid = self.start_server(self.args)
+        orig_m2_ssl_free = SSL.Connection.m2_ssl_free
+        def _m2_ssl_free(ssl):
+            orig_m2_ssl_free(ssl)
+            _m2_ssl_free.called = True
+
+        try:
+            ctx = SSL.Context()
+            s = SSL.Connection(ctx)
+            s.m2_ssl_free = _m2_ssl_free
+            s.connect(self.srv_addr)
+            data = self.http_get(s)
+            s.close()
+            self.assertFalse(hasattr(_m2_ssl_free, 'called'))
+            # keep fingers crossed that SSL.Connection.__del__ is called
+            # by the python interpreter
+            del s
+        finally:
+            self.stop_server(pid)
+        self.assertIn('s_server -quiet -www', data)
+        self.assertTrue(getattr(_m2_ssl_free, 'called', False))
+
+    def test_ssl_connection_no_free(self):
+        pid = self.start_server(self.args)
+        orig_m2_ssl_free = SSL.Connection.m2_ssl_free
+        def _m2_ssl_free(ssl):
+            _m2_ssl_free.called = True
+            orig_m2_ssl_free(ssl)
+
+        try:
+            ctx = SSL.Context()
+            s = SSL.Connection(ctx)
+            s.m2_ssl_free = _m2_ssl_free
+            s.set_ssl_close_flag(m2.bio_close)
+            s.connect(self.srv_addr)
+            data = self.http_get(s)
+            s.close()
+            self.assertFalse(hasattr(_m2_ssl_free, 'called'))
+            # keep fingers crossed that SSL.Connection.__del__ is called
+            # by the python interpreter
+            del s
+        finally:
+            self.stop_server(pid)
+        self.assertIn('s_server -quiet -www', data)
+        self.assertFalse(hasattr(_m2_ssl_free, 'called'))
+
 
 class UrllibSSLClientTestCase(BaseSSLClientTestCase):
 
@@ -981,7 +1001,28 @@ class Urllib2SSLClientTestCase(BaseSSLClientTestCase):
             self.stop_server(pid)
 
 
-@unittest.skipUnless(util.py27plus,
+class Urllib2TEChunkedSSLClientTestCase(BaseSSLClientTestCase):
+    """Test a response with "Transfer-Encoding: chunked"."""
+
+    def setUp(self):
+        super(Urllib2TEChunkedSSLClientTestCase, self).setUp()
+        self.args = ['s_server', '-quiet', '-HTTP',
+                     '-accept', str(self.srv_port)]
+
+    def test_transfer_encoding_chunked(self):
+        pid = self.start_server(self.args)
+        try:
+            url = 'https://%s:%s/te_chunked_response.txt' % (srv_host,
+                                                             self.srv_port)
+            o = m2urllib2.build_opener()
+            u = o.open(url)
+            data = u.read()
+            self.assertEqual(b'foo\nfoobar\n', data)
+        finally:
+            self.stop_server(pid)
+
+
+@unittest.skipUnless(py27plus,
                      "Twisted doesn't test well with Python 2.6")
 class TwistedSSLClientTestCase(BaseSSLClientTestCase):
 
@@ -1085,7 +1126,7 @@ class TwistedSSLClientTestCase(BaseSSLClientTestCase):
 
         pid = self.start_server(self.args)
 
-        class ContextFactory:
+        class ContextFactory(object):
             def getContext(self):
                 return SSL.Context()
 
@@ -1143,10 +1184,11 @@ def suite():
     suite.addTest(unittest.makeSuite(HttpslibSSLSNIClientTestCase))
     suite.addTest(unittest.makeSuite(UrllibSSLClientTestCase))
     suite.addTest(unittest.makeSuite(Urllib2SSLClientTestCase))
+    suite.addTest(unittest.makeSuite(Urllib2TEChunkedSSLClientTestCase))
     suite.addTest(unittest.makeSuite(MiscSSLClientTestCase))
     suite.addTest(unittest.makeSuite(FtpslibTestCase))
     try:
-        if util.py27plus:
+        if py27plus:
             import M2Crypto.SSL.TwistedProtocolWrapper as wrapper  # noqa
             suite.addTest(unittest.makeSuite(TwistedSSLClientTestCase))
     except ImportError:
